@@ -8,6 +8,7 @@ import (
 	"strings"
 	"syscall"
 	"unsafe"
+	gotime "time"
 
 	"golang.org/x/sys/unix"
 	"modernc.org/libc/fcntl"
@@ -22,6 +23,10 @@ type (
 	ulong = uint64
 )
 
+var (
+	startTime = gotime.Now() // For clock(3)
+)
+
 // int sigaction(int signum, const struct sigaction *act, struct sigaction *oldact);
 func Xsigaction(t *TLS, signum int32, act, oldact uintptr) int32 {
 	if __ccgo_strace {
@@ -709,3 +714,11 @@ func Xopendir(t *TLS, name uintptr) uintptr {
 	(*darwinDir)(unsafe.Pointer(p)).eof = false
 	return p
 }
+
+// clock_t clock(void);
+func Xclock(t *TLS) time.Clock_t {
+	if __ccgo_strace {
+		trc("t=%v, (%v:)", t, origin(2))
+	}
+	return time.Clock_t(gotime.Since(startTime) * gotime.Duration(time.CLOCKS_PER_SEC) / gotime.Second)
+}
